const Main = imports.ui.main;
const Config = imports.misc.config;
const Meta = imports.gi.Meta;
const St = imports.gi.St;
const Lang = imports.lang;

var _version;

var _transparentPanel;

const TransparentPanel = new Lang.Class({
    Name: 'TransparentPanel',

    _init()
    {
        this.actor = Main.panel;

        this._signalIds = new Map();
        this._trackedWindows = new Map();

        this._signalIds.set(Main.overview, [
            Main.overview.connect('showing', this._updateSolidStyle.bind(this)),
            Main.overview.connect('hiding', this._updateSolidStyle.bind(this))
        ]);

        this._signalIds.set(Main.sessionMode, [
            Main.sessionMode.connect('updated', this._updateSolidStyle.bind(this))
        ]);

        for (const metaWindowActor of global.get_window_actors())
            this._onWindowActorAdded(metaWindowActor.get_parent(), metaWindowActor);

        this._signalIds.set(global.window_group, [
            global.window_group.connect('actor-added', this._onWindowActorAdded.bind(this)),
            global.window_group.connect('actor-removed', this._onWindowActorRemoved.bind(this))
        ]);

        this._signalIds.set(global.window_manager, [
            global.window_manager.connect('switch-workspace', this._updateSolidStyle.bind(this))
        ]);

        this._updateSolidStyle();
    },

    destroy()
    {
        for (const allSignalIds of [this._signalIds, this._trackedWindows])
            for (const [actor, signalIds] of allSignalIds)
                for (const signalId of signalIds)
                    actor.disconnect(signalId);
    },

    _onWindowActorAdded(container, metaWindowActor)
    {
        let signalIds = [];
        [_version < 38 ? 'allocation-changed' : 'notify::allocation', 'notify::visible'].forEach(s => {
            signalIds.push(metaWindowActor.connect(s, this._updateSolidStyle.bind(this)));
        });
        this._trackedWindows.set(metaWindowActor, signalIds);
    },

    _onWindowActorRemoved(container, metaWindowActor)
    {
        this._trackedWindows.get(metaWindowActor).forEach(id => {
            metaWindowActor.disconnect(id);
        });
        this._trackedWindows.delete(metaWindowActor);
        this._updateSolidStyle();
    },

    _updateSolidStyle() {
        if (Main.panel.has_style_pseudo_class('overview') || !Main.sessionMode.hasWindows)
        {
            Main.panel._addStyleClassName('transparent');
            return;
        }

        if (!Main.layoutManager.primaryMonitor)
            return;

        /* Get all the windows in the active workspace that are in the primary monitor and visible */
        let workspaceManager = global.workspace_manager;
        let activeWorkspace = workspaceManager.get_active_workspace();
        let windows = activeWorkspace.list_windows().filter(metaWindow => {
            return metaWindow.is_on_primary_monitor() &&
                   metaWindow.showing_on_its_workspace() &&
                   !metaWindow.is_hidden() &&
                   metaWindow.get_window_type() != Meta.WindowType.DESKTOP;
        });

        /* Check if at least one window is near enough to the panel */
        let [, panelTop] = this.actor.get_transformed_position();
        let panelBottom = panelTop + this.actor.get_height();
        let scale = St.ThemeContext.get_for_stage(global.stage).scale_factor;
        let isNearEnough = windows.some(metaWindow => {
            let verticalPosition = metaWindow.get_frame_rect().y;
            return verticalPosition < panelBottom + 5 * scale;
        });


        if (isNearEnough)
            Main.panel._removeStyleClassName('transparent');
        else
            Main.panel._addStyleClassName('transparent');
    }
});

function init()
{
    _version = parseInt(Config.PACKAGE_VERSION.split('.')[1]);
}

function enable()
{
    Main.panel._addStyleClassName('transparent');

    if (_version > 30)
        _transparentPanel = new TransparentPanel();
}

function disable()
{
    if (_version > 30)
        _transparentPanel.destroy();

    Main.panel._removeStyleClassName('transparent');
}
